import { Body, Delete, Get, JsonController, OnUndefined, Param, Post, Put, QueryParam } from 'routing-controllers';
import { OpenAPI, ResponseSchema } from 'routing-controllers-openapi';
import { getConnectionManager, Repository } from 'typeorm';
import { EntityFromBody } from 'typeorm-routing-controllers-extensions';
import { UserIdsNotMatchingError, UserNotFoundError } from '../errors/UserErrors';
import { UserGroupNotFoundError } from '../errors/UserGroupErrors';
import { CreateUser } from '../models/actions/CreateUser';
import { User } from '../models/entities/User';
import { ResponseEmpty } from '../models/responses/ResponseEmpty';
import { ResponseUser } from '../models/responses/ResponseUser';


@JsonController('/users')
export class UserController {
	private userRepository: Repository<User>;

	/**
	 * Gets the repository of this controller's model/entity.
	 */
	constructor() {
		this.userRepository = getConnectionManager().get().getRepository(User);
	}

	@Get()
	@ResponseSchema(User, { isArray: true })
	@OpenAPI({ description: 'Lists all users.' })
	async getAll() {
		let responseUsers: ResponseUser[] = new Array<ResponseUser>();
		const users = await this.userRepository.find({ relations: ['permissions', 'groups'] });
		users.forEach(user => {
			responseUsers.push(new ResponseUser(user));
		});
		return responseUsers;
	}

	@Get('/:id')
	@ResponseSchema(User)
	@ResponseSchema(UserNotFoundError, { statusCode: 404 })
	@OnUndefined(UserNotFoundError)
	@OpenAPI({ description: 'Returns a user of a specified id (if it exists)' })
	async getOne(@Param('id') id: number) {
		let user = await this.userRepository.findOne({ id: id }, { relations: ['permissions', 'groups'] })
		if (!user) { throw new UserNotFoundError(); }
		return new ResponseUser(user);
	}

	@Post()
	@ResponseSchema(User)
	@ResponseSchema(UserGroupNotFoundError)
	@OpenAPI({ description: 'Create a new user object (id will be generated automagicly).' })
	async post(@Body({ validate: true }) createUser: CreateUser) {
		let user;
		try {
			user = await createUser.toUser();
		} catch (error) {
			throw error;
		}

		user = await this.userRepository.save(user)
		return new ResponseUser(await this.userRepository.findOne(user, { relations: ['permissions', 'groups'] }));
	}

	@Put('/:id')
	@ResponseSchema(User)
	@ResponseSchema(UserNotFoundError, { statusCode: 404 })
	@ResponseSchema(UserIdsNotMatchingError, { statusCode: 406 })
	@OpenAPI({ description: "Update a user object (id can't be changed)." })
	async put(@Param('id') id: number, @EntityFromBody() user: User) {
		let oldUser = await this.userRepository.findOne({ id: id }, { relations: ['permissions', 'groups'] });

		if (!oldUser) {
			throw new UserNotFoundError();
		}

		if (oldUser.id != user.id) {
			throw new UserIdsNotMatchingError();
		}

		await this.userRepository.update(oldUser, user);
		return new ResponseUser(await this.userRepository.findOne({ id: id }, { relations: ['permissions', 'groups'] }));
	}

	@Delete('/:id')
	@ResponseSchema(User)
	@ResponseSchema(ResponseEmpty, { statusCode: 204 })
	@OnUndefined(204)
	@OpenAPI({ description: 'Delete a specified runner (if it exists).' })
	async remove(@Param("id") id: number, @QueryParam("force") force: boolean) {
		let runner = await this.userRepository.findOne({ id: id });
		if (!runner) { return null; }
		const responseUser = await this.userRepository.findOne(runner, { relations: ['permissions', 'groups'] });

		if (!runner) {
			throw new UserNotFoundError();
		}

		await this.userRepository.delete(runner);
		return new ResponseUser(responseUser);
	}
}
